from mpmath import mp, mpf, sqrt
import time

def bs_pi(digits):
	#Calculates pi to precision = digits using Brent-Salamin's algorithm
	mp.dps = digits					#set mpmath precision to digits
	eps = 1/mpf(10**digits)			#set cut-off limit to break loop later
	a = mpf(1)						#Initialize a in the AGM
	b = 1/sqrt(mpf(2))				#Initialize b in the AGM
	diff = a - b
	series = mpf(0)					#Initialize sum
	t = time.time()
	
	n = 0							#Setting up counter for number of iterations
	#Iteration of AGM
	while diff > eps:
		n +=1
		arm = (a + b)/2				#get arithmetic mean
		geom = sqrt(a*b)			#get geometric mean
		a, b = arm, geom			#reassign a,b to arith and geom mean, respectively
		series += 2**(n+1) * (a**2 - b**2)	#sum to series on the bottom part
		diff = a - b				#compute diff to check if algorithm has reached threshhold
	# after a and b converges:
	pie = (4*a*a)/(1 - series)
	t = time.time() - t
	print "Used %d steps to calculate pi to %d digits, a is %r and b is %r" % (n, digits, a, b)
	return pie, t

	
if __name__ == "__main__":
    for i in range(1,9):
        digits = 10**i
        pi, t = bs_pi(digits)
        print("chudnovsky: digits",digits,"time:",t)
		
# Used 7 steps to calculate pi to 100 digits
# ('chudnovsky: digits', 100, 'time:', 0.0009999275207519531)
# Used 10 steps to calculate pi to 1000 digits
# ('chudnovsky: digits', 1000, 'time:', 0.0009999275207519531)
# Used 13 steps to calculate pi to 10000 digits
# ('chudnovsky: digits', 10000, 'time:', 0.004999876022338867)
# Used 17 steps to calculate pi to 100000 digits
# ('chudnovsky: digits', 100000, 'time:', 0.18199992179870605)
# Used 20 steps to calculate pi to 1000000 digits
# ('chudnovsky: digits', 1000000, 'time:', 2.8550000190734863)
# Used 23 steps to calculate pi to 10000000 digits
# ('chudnovsky: digits', 10000000, 'time:', 44.75099992752075)
# Used 27 steps to calculate pi to 100000000 digits
# ('chudnovsky: digits', 100000000, 'time:', 706.2420001029968) ~100 mil
