from time import time

def atan_euler(x, one):
	#Euler's accelerated version of Taylor series expansion of arctan that calculates arctan(1/x)*10**one
	#i.e gives an integer representation of arctan(1/x)
	xsq = x * x  						#Preload x**2
	xsq_plus1 = xsq + 1					#Preload x**2 + 1
	term = (x * one) // xsq_plus1		#Initialize first term multiplied by 10**digits
	total = term						#Initalize the series sum
	two_n = 2							#2n factor to be changed each term
	while 1:
		divisor = (two_n + 1) * xsq_plus1
		#Calculate each term
		term *= two_n
		term += divisor // 2 
		term = term // divisor
		if term == 0:
			break						#breaks when term < divisor
		total += term
		two_n += 2
	return total
	
def pi_machin(one):
	#Machin's formula pi = 4*(4arctan(1/5) + arctan(1/239))
	#one = 10**digits
    return 4*(4*atan_euler(5, one) - atan_euler(239, one))
	
def pi_gauss_euler(one):
	#Machin's formula pi = 4*(12arctan(1/18) + *arctan(1/57) - 5arctan(1/239))
	#one = 10**digits
    return 4*(12*atan_euler(18, one) + 8*atan_euler(57, one) - 5*atan_euler(239, one))

#returns pi with 10 to 10^6 digits, increasing by a factor of 10 each time
if __name__ == "__main__":
	for i in range(1,7):
		digits = 10**i
		one = 10**digits
		
		t = time()
		pie = pi_machin(one)
		print "Machin:", digits, "digits; time:", time() - t
		
		t = time()
		pie = pi_gauss_euler(one)
		print "GaussE:", digits, "digits; time:", time() - t
		
# Machin: 10 digits; time: 0.000999927520752
# GaussE: 10 digits; time: 0.000999927520752
# Machin: 100 digits; time: 0.0
# GaussE: 100 digits; time: 0.0
# Machin: 1000 digits; time: 0.00200009346008
# GaussE: 1000 digits; time: 0.00200009346008
# Machin: 10000 digits; time: 0.0980000495911
# GaussE: 10000 digits; time: 0.0939998626709
# Machin: 100000 digits; time: 8.88899993896
# GaussE: 100000 digits; time: 8.69600009918
# Machin: 1000000 digits; time: 977.108999968
# GaussE: 1000000 digits; time: 976.340999842